#include "util.h"

#include <avr/interrupt.h>
#include <avr/io.h>

#include <util/delay.h>

#include <stdio.h>
#include <stdlib.h>

volatile uint8_t countdown_timer = 0;
volatile uint8_t enable_timer = 0;
volatile uint8_t enable_alarm = 0;

/* 
 TODO 2: implementati rutinele de tratare a intreruperii de butoane.
 Cele doua butoane sunt asociate unor intreruperi diferite
 */
ISR(PCINT1_vect)
{
    // TODO 2: Verificati daca PB2 este apasat si modificati starea LED-ului conectat la PB3.
    if ((PINB & (1 << PB2)) == 0)
    {
        // TODO 1: aprindeti LED-ul legat la PB3 la apasarea lui PB2
        PORTB |= (1 << PB3);
        // TODO 2: activare cronometru
        countdown_timer = 10;
        enable_timer = 1;
    }
}

ISR(PCINT3_vect)
{
    // TODO 2: Verificati daca PD6 este apasat si modificati starea LED-ului conectat la PB3.
    if ((PIND & (1 << PD6)) == 0)
    {
        // TODO 1: stingeti LED-ul legat la PB3 la apasarea lui PD6
        PORTB &= ~(1 << PB3);
        // TODO 2: dezactivare cronometru
        enable_timer = 0;
        countdown_timer = 0;
        enable_alarm = 0;
    }
}

/* intreruperea de comparatie cu OCR1A */
ISR(TIMER1_COMPA_vect)
{
    // configurati timer1 pentru a genera întreruperi la fiecare secundă
    if (enable_timer)
    {
        // TODO 3: LED care se aprinde la fiecare secunda, timp de 200 ms
        PORTB |= (1 << PB3);
        if (countdown_timer > 0)
        {
            countdown_timer -= 1;
        }
        else
        {
            enable_alarm = 1;
        }
    }
}

// TODO 4: implementati rutina de tratare a intreruperii de comparatie cu OCR1B.
ISR(TIMER1_COMPB_vect)
{
    if (enable_timer)
    {
        // TODO 3: LED care se aprinde la fiecare secunda, timp de 200 ms
        PORTB &= ~(1 << PB3);
    }
}

// TODO 4: implementati rutina de tratare a intreruperii de comparatie cu OCR2A.
ISR(TIMER2_COMPA_vect)
{
    if (enable_alarm)
    {
        // TODO 4: alarma pentru cronometrul implementat anterior
        PORTD ^= (1 << PD4);
    }
}

/* initializeaza intrarile si iesirile */
void IO_init()
{
    // LED-ul verde este pus ca iesire
    DDRD |= (1 << PD7);

    // backlight LCD
    DDRC |= (1 << PC7);
    PORTC |= (1 << PC7);

    // TODO 1: Initializati LED-ul conectat la PB3 ca iesire.
    DDRB |= (1 << PB3);

    // TODO 3: Initializati buzzer-ul conectat la PD4 ca iesire.
    DDRD |= (1 << PD4);

    // butoanele sunt puse ca intrare, cu rezistenta de pull-up
    DDRB &= ~(1 << PB2);
    PORTB |= (1 << PB2);

    DDRD &= ~(1 << PD6);
    PORTD |= (1 << PD6);

    // TODO 1: Activati intreruperile pe butoane

    // Activez PIN CHANGE INTERRUPT ENABLE PCIE1 pentru PCINT15:8 (portul B) si PCIE3 pentru PCINT24:31 (portul D)
    PCICR |= (1 << PCIE1) | (1 << PCIE3);
    // Activez intreruperea pe butoanele PB2 si PD6
    PCMSK1 |= (1 << PCINT10);
    PCMSK3 |= (1 << PCINT30);
}

void timer2_init()
{
    // CTC cu top la OCR2A
    TCCR2A |= (1 << WGM22);
    // Prescaler 64
    TCCR2B |= (1 << CS22);
    // f_int = 1250Hz
    OCR2A = 149;
    // Activeaza intreruperea de comparare cu OCR2A
    TIMSK2 |= (1 << OCIE2A);
}

/* initializeaza timer-ul 1 */
void timer1_init()
{
    // CTC cu top la OCR1A
    TCCR1B = (1 << WGM12);
    // Prescaler 256
    TCCR1B |= (1 << CS12);
    // f_int = 1hz
    OCR1A = 46875;
    // on 200 ms (1/5s)
    OCR1B = OCR1A / 5;
    // Activeaza intreruperea de comparare cu OCR1A și OCR1B
    TIMSK1 = (1 << OCIE1A) | (1 << OCIE1B);
}

/* TODO 1: explicati modul de functionare al timer-ului */
void timer1_init_exemplu()
{
    // configurare limita counter: 4 secunde
    ICR1 = 46875;
    OCR1A = 46875;
    // configurare mod functionare pentru Timer1: CTC cu top la ICR1
    TCCR1B |= (1 << WGM13) | (1 << WGM12);
    // configurare prescaler: 1024
    TCCR1B |= (1 << CS12) | (1 << CS10);
}

void print_status()
{
    if (enable_timer)
    {
        LCD_printAt(0x00, "Timer activat");
        char buf[9];
        snprintf(buf, sizeof(buf), "%02d", countdown_timer);
        LCD_printAt(0x40, buf);
    }
    else
    {
        LCD_printAt(0x00, "Timer dezactivat");
    }
}

int main(void)
{
    // initializari si testare interfata afisare/debug
    UI_init();

    _delay_ms(1000);

    // codul relevant laboratorului incepe aici
    sei();

    timer1_init();
    timer2_init();
    IO_init();

    uint8_t mode = 0;

    while (1)
    {
        switch (mode)
        {
        case 0:
            drawProgressBars(TCNT1, OCR1A, 0xFFFF);
            break;
        case 1:
            print_status();
            break;
        }
        _delay_ms(100);
    }

    return 0;
}
